"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObservableExecutorDecorator = void 0;
class ObservableExecutorDecorator {
    constructor(expressions, executor, helper) {
        this.expressions = expressions;
        this.executor = executor;
        this.helper = helper;
        this.observers = [];
    }
    execute(query, data, options) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let result = null;
                yield this.beforeExecutionNotify(query, data, options);
                if (this.observers.find(p => p.transactional === true) !== undefined) {
                    // execute before and after listeners transactional
                    this.executor.transaction(options, (tr) => __awaiter(this, void 0, void 0, function* () {
                        yield this.beforeExecutionNotify(query, data, options, true);
                        result = yield tr.execute(query, data);
                        yield this.afterExecutionNotify(query, data, options, result, true);
                    }));
                }
                else {
                    result = yield this.executor.execute(query, data, options);
                }
                yield this.afterExecutionNotify(query, data, options, result);
                return result;
            }
            catch (error) {
                yield this.errorExecutionNotify(query, data, options, error);
                throw error;
            }
        });
    }
    executeList(queries, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.executor.executeList(queries, options);
        });
    }
    transaction(options, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.executor.transaction(options, callback);
        });
    }
    // Listeners and subscribers
    subscribe(observer) {
        this.observers.push(observer);
    }
    unsubscribe(observer) {
        const index = this.observers.indexOf(observer);
        if (index === -1) {
            throw new Error('Subject: Nonexistent observer.');
        }
        this.observers.splice(index, 1);
    }
    beforeExecutionNotify(query, data, options, transactional = false) {
        return __awaiter(this, void 0, void 0, function* () {
            const args = { query, data, options };
            this.observers.filter(p => p.actions.includes(query.action) && this.helper.utils.nvl(p.transactional, false) === transactional).forEach((observer) => __awaiter(this, void 0, void 0, function* () {
                if (observer.condition === undefined) {
                    observer.before(args);
                }
                else {
                    const context = { query, options };
                    if (this.expressions.eval(observer.condition, context)) {
                        yield observer.before(args);
                    }
                }
            }));
        });
    }
    afterExecutionNotify(query, data, options, result, transactional = false) {
        return __awaiter(this, void 0, void 0, function* () {
            const args = { query, data, options, result };
            this.observers.filter(p => p.actions.includes(query.action) && this.helper.utils.nvl(p.transactional, false) === transactional).forEach((observer) => __awaiter(this, void 0, void 0, function* () {
                if (observer.condition === undefined) {
                    observer.after(args);
                }
                else {
                    const context = { query, options };
                    if (this.expressions.eval(observer.condition, context)) {
                        yield observer.after(args);
                    }
                }
            }));
        });
    }
    errorExecutionNotify(query, data, options, error) {
        return __awaiter(this, void 0, void 0, function* () {
            const args = { query, data, options, error };
            this.observers.filter(p => p.actions.includes(query.action)).forEach((observer) => __awaiter(this, void 0, void 0, function* () {
                if (observer.condition === undefined) {
                    observer.error(args);
                }
                else {
                    const context = { query, options };
                    if (this.expressions.eval(observer.condition, context)) {
                        yield observer.error(args);
                    }
                }
            }));
        });
    }
}
exports.ObservableExecutorDecorator = ObservableExecutorDecorator;
//# sourceMappingURL=observableExecutorDecorator.js.map